<?php

declare(strict_types=1);

namespace App\Services\WorkingDays\PublicHolidays\Storage;

use App\Models\Country;
use App\Models\NonWorkingDays;
use Illuminate\Database\UniqueConstraintViolationException;
use Illuminate\Support\Facades\Log;

class DatabaseStorage implements PublicHolidaysStorageInterface
{
    public function isPublicHoliday(string $countryCode, \DateTimeImmutable $holidayDate): bool
    {
        $country = $this->getCountryByCountryCode($countryCode);

        return NonWorkingDays::where('country_id', $country->id)
            ->where('non_working_date', $holidayDate->format('Y-m-d'))
            ->exists();
    }

    public function storePublicHoliday(string $countryCode, \DateTimeImmutable $holidayDate): ?NonWorkingDays
    {
        $country = $this->getCountryByCountryCode($countryCode);

        try {
            return NonWorkingDays::create([
                'country_id' => $country->id,
                'non_working_date' => $holidayDate->format('Y-m-d'),
            ]);
        } catch (UniqueConstraintViolationException $e) {
            // safe to ignore ... combination exists
            Log::warning("Non-working day already exists for country: " . $country . ", date: {$holidayDate->format('Y-m-d')}");
        }

        return null;
    }

    public function getPublicHolidaysInInterval(string $countryCode, \DateTimeImmutable $startDate, \DateTimeImmutable $endDate): array
    {
        $country = $this->getCountryByCountryCode($countryCode);
        return NonWorkingDays::where('country_id', $country->id)
           ->whereBetween('non_working_date', [$startDate->format('Y-m-d'), $endDate->format('Y-m-d')])
           ->get()->all();
    }


    public function countPublicHolidaysInInterval(string $countryCode, \DateTimeImmutable $startDate, \DateTimeImmutable $endDate): int
    {
        $country = $this->getCountryByCountryCode($countryCode);
        return NonWorkingDays::where('country_id', $country->id)
            ->whereBetween('non_working_date', [$startDate->format('Y-m-d'), $endDate->format('Y-m-d')])
            ->count();
    }

    protected function getCountryByCountryCode(string $countryCode): Country
    {
        return Country::where('country_code', $countryCode)->first() ?? throw new \InvalidArgumentException("Country code '$countryCode' does not exist.");
    }
}
